#include <ctype.h>
#include "extchar.h"

/*
Funcoes de manipulacao de caracteres utilizando unsigned char

Tabela de caracteres:
http://en.wikipedia.org/wiki/Code_page_850
*/

/* dado um caractere, retorna o a versao dele em minuscula */
unsigned char tolowerextch(unsigned char c)
{
    if(c < 128)
        return tolower(c);
    else
        switch(c)
        {
            case 128: return 135;// (C_cedil)
            case 142: return 132;// (A_uml)
            case 143: return 134;// (A_ring)
            case 144: return 130;// (C_acute)
            case 153: return 148;// (O_uml)
            case 154: return 129;// (U_uml)
            case 165: return 164;// (N_tilde)
            case 181: return 160;// (A_acute)
            case 182: return 131;// (A_circ)
            case 183: return 133;// (A_grave)
            case 199: return 198;// (A_tilde)
            case 210: return 136;// (E_circ)
            case 211: return 137;// (E_uml)
            case 212: return 138;// (E_grave)
            case 214: return 161;// (I_acute)
            case 215: return 140;// (I_circ)
            case 216: return 139;// (I_uml)
            case 222: return 141;// (I_grave)
            case 224: return 162;// (O_acute)
            case 226: return 147;// (O_circ)
            case 229: return 228;// (O_tilde)
            case 233: return 163;// (U_acute)
            case 234: return 150;// (U_circ)
            case 235: return 151;// (U_grave)
            case 237: return 236;// (Y_acute)
            default: return c;
        }
}

/* dado um caractere, retorna a versao dele sem acentuacao (cedilha vira c)

Tabela de caracteres:
http://en.wikipedia.org/wiki/Code_page_850
*/
unsigned char removeaccentsch(unsigned char c)
{
    switch(c)
    {
        case 181: return 65;// (A)
        case 182: return 65;// (A)
        case 183: return 65;// (A)
        case 199: return 65;// (A)
        case 142: return 65;// (A)
        case 143: return 65;// (A)
        case 198: return 97;// (a)
        case 131: return 97;// (a)
        case 132: return 97;// (a)
        case 133: return 97;// (a)
        case 134: return 97;// (a)
        case 160: return 97;// (a)

        case 144: return 69;// (E)
        case 210: return 69;// (E)
        case 211: return 69;// (E)
        case 212: return 69;// (E)
        case 130: return 101;// (e)
        case 136: return 101;// (e)
        case 137: return 101;// (e)
        case 138: return 101;// (e)

        case 214: return 73;// (I)
        case 215: return 73;// (I)
        case 216: return 73;// (I)
        case 222: return 73;// (I)
        case 213: return 105;// (i)
        case 139: return 105;// (i)
        case 140: return 105;// (i)
        case 141: return 105;// (i)
        case 161: return 105;// (i)
        case 163: return 105;// (i)

        case 229: return 79;// (O)
        case 224: return 79;// (O)
        case 226: return 79;// (O)
        case 227: return 79;// (O)
        case 153: return 79;// (O)
        case 147: return 111;// (o)
        case 148: return 111;// (o)
        case 149: return 111;// (o)
        case 162: return 111;// (o)
        case 228: return 111;// (o)

        case 233: return 85;// (U)
        case 234: return 85;// (U)
        case 235: return 85;// (U)
        case 154: return 85;// (U)
        case 129: return 117;// (u)
        case 150: return 117;// (u)
        case 151: return 117;// (u)

        case 165: return 78;// (N)
        case 164: return 110;// (n)

        case 128: return 67;// (C)
        case 135: return 99;// (c)

        case 237: return 89;// (Y)
        case 152: return 121;// (y)
        case 236: return 121;// (y)

        default: return c;
    }
}
