#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include "common.h"
#include "tad_dicionario.h"
#include "tad_palavra.h"
#include "ansi_to_ascii.h"

/* inicializa o dicionario (ABP) */
PtDic* incializa_dic(void)
{
    return NULL;
}

/* Importa as palavras de um dicionario para a arvore */
PtDic* ler_dic(PtDic* d, char dic[])
{
    TipoPalavra buffer[1024];
    TipoPalavra palavra[MAX];
    FILE *p;
    char pasta[]= "dicionarios/";
    int num_palavras=0, i=0, i_buffer=0, n_letras_lidas;
    time_t start,end;

    //abre o arquivo conforme a escolha realizada
    p=fopen(strcat(pasta,dic),"r");

    // verifica se abriu o arquivo corretamente
    if (p != NULL)
    {
        //inicializa a palavra temporaria
        cria_palavra(palavra);

        //numero de palavras no arquivo
        num_palavras = conta_palavras_arq(p);

        //insere palavras no dicionario
        time(&start);
        while (!feof(p))
        {
            //le letras do arquivocarmazenando-as em um buffer
            n_letras_lidas = fread(buffer,sizeof(unsigned char),sizeof(buffer),p);

            //passa caractere por caractere para as palavras e entao para o dicionario
            for(i_buffer=0; i_buffer < n_letras_lidas; i_buffer++)
            {
                //verifica se era caractere valido
                if(verifica_letra(buffer[i_buffer]))
                {
                    insere_letra(palavra, buffer[i_buffer]);
                }
                //caractere invalido e ja tinha inicializado uma palavra, logo, encerrou uma palavra
                else if(strlen(palavra) > 0)
                {
                    //insere a palavra no dicionario
                    d = insere_palavra_dic(d, palavra);

                    //reinicia a palavra temporaria
                    cria_palavra(palavra);

                    //incrementa contador de palavras na arvore
                    i++;

                    time (&end);

                    /*
                    //usado para indicar a porcentagem do carregamento
                    if(i%1000 == 0)
                    {
                        printf("%.2f%% (%d/%d) | Alt: %2d | Tempo: %.0lfs",((float)i/num_palavras)*100.0,i,num_palavras,calc_altura(d),difftime(end,start));
                        if(difftime(end,start)>0)
                            printf(" |  Veloc: %.2lf pal/seg\n",i/difftime(end,start));
                        else
                            printf("\n");
                    }
                    */
                }
            }
        }

        //verifica se a ultima palavra ficou pendente por ter encontrado o EOF
        if(strlen(palavra)>0)
        {
            //insere a palavra no dicionario
            d = insere_palavra_dic(d, palavra);

            //incrementa contador de palavras inseridos na arvore
            i++;
        }

        time (&end);
        /*
        //usado para indicar a porcentagem do carregamento
        printf("%.2f%% (%d/%d) | Alt: %2d | Tempo: %.0lfs",((float)i/num_palavras)*100.0,i,num_palavras,calc_altura(d),difftime(end,start));
        if(difftime(end,start)>0)
            printf(" |  Veloc: %.2lf pal/seg\n",i/difftime(end,start));
        else
            printf("\n");

        printf("\n\nPalavras inseridas: %d\n",i);
        */

        fclose(p);
    }
    else
    {
        printf("Erro ao abrir o arquivo!\n");
    }

    return d;
}

/* conta o numero de nodos de uma arvore */
int conta_nodos(PtDic *d)
{
    int nodos_dir, nodos_esq;

    if(d == NULL)
    {
        return 0;
    }
    else
    {
        nodos_esq = conta_nodos(d->esq);
        nodos_dir = conta_nodos(d->dir);

        return 1 + nodos_esq + nodos_dir;
    }
}

/* dada um nodo, calcula a sua altura */
int calc_altura(PtDic *a)
{
    int Calc_Esq, Calc_Dir;

    if (a != NULL)
    {
        Calc_Esq = calc_altura(a->esq);
        Calc_Dir = calc_altura(a->dir);

        if (Calc_Esq > Calc_Dir)
            return (1 + Calc_Esq);
        else
            return (1 + Calc_Dir);
    }

    return 0;
}

/*Dado um palavra e um dicionario, verifica se essa palavra est no dicionrio*/
int pesquisa_palavra (PtDic *d, char pal[MAX])
{
    //rvore vazia
    if (d==NULL)
    {
        return 0;
    }
    //Achou a palavra
    else if (palavracmp(d->palavra,pal)==0)
    {
        return 1;
    }
    //No encontrou a palavra e a rvore no est vazia
    else
    {
        //Palavra procurada  menor, segue busca no nodo esquerdo
        if (palavracmp(d->palavra,pal)>0)
            return pesquisa_palavra(d->esq,pal);
        //Palavra procurada  maior, segue busca no nodo direito
        else if (palavracmp(d->palavra,pal)<0)
            return pesquisa_palavra(d->dir,pal);
    }
}

/*Calcula Fator para balancear*/
int calc_fator(PtDic *d)
{
    return calc_altura(d->esq) - calc_altura(d->dir);
}

/* verifica se a arvore esta balanceada */
int verifica_balancemento(PtDic *d)
{
    int alt_esq,alt_dir;

    if (d!=NULL)
    {
        alt_esq=calc_altura(d->esq);
        alt_dir=calc_altura(d->dir);
        return ((alt_esq - alt_dir <2) && (alt_dir - alt_esq <2) && (verifica_balancemento(d->esq)) && (verifica_balancemento(d->dir)));
    }
    else
        return 1;
}

/*Rotao Simples a direita*/
PtDic* rotacao_direita(PtDic *d)
{
    PtDic *ptu;
    int d_dir, d_esq, ptu_esq;

    ptu = d->esq;
    d->esq = ptu->dir;
    ptu->dir = d;

    //atualiza as alturas dos nodos
    d_dir = get_altura ( d->dir );
    d_esq = get_altura ( d->esq );
    ptu_esq = get_altura ( ptu->esq );

    //altura de d eh a maior das suas alturas mais o seu nodo
    d->altura = (d_dir > d_esq ? d_dir : d_esq) + 1;
    //altura de ptu eh a maior das suas alturas mais o seu nodo
    ptu->altura = (ptu_esq > d->altura ? ptu_esq : d->altura ) + 1;

    return ptu;
}

/*Rotao Simples a esquerda*/
PtDic* rotacao_esquerda(PtDic *d)
{
    PtDic *ptu;
    int d_dir, d_esq, ptu_dir;

    ptu = d->dir;
    d->dir = ptu->esq;
    ptu->esq = d;

    //atualiza as alturas dos nodos
    d_dir = get_altura ( d->dir );
    d_esq = get_altura ( d->esq );
    ptu_dir = get_altura ( ptu->dir );

    //altura de d eh a maior das suas alturas mais o seu nodo
    d->altura = (d_dir > d_esq ? d_dir : d_esq) + 1;
    //altura de ptu eh a maior das suas alturas mais o seu nodo
    ptu->altura = (ptu_dir > d->altura ? ptu_dir : d->altura ) + 1;

    return ptu;
}

/*Rotao Dupla a direita*/
PtDic* rotacao_dupla_direita (PtDic *d)
{
    /*
    Como a rotacao dupla direita eh simplesmente uma rotacao esquerda no
    nodo da esquerda da raiz da subarvore desbalanceada seguida de uma rotacao
    direita no nodo raiz da subarv. desbalanceada, fica mais simples definir essa
    funcao utilizando as funcoes ja criadas:
    */
    d->esq = rotacao_esquerda(d->esq);
    return rotacao_direita(d);
}

/*Rotao Dupla a esquerda*/
PtDic* rotacao_dupla_esquerda (PtDic *d)
{
    /*
    Como a rotacao dupla esquerda eh simplesmente uma rotacao direita no
    nodo da direita da raiz da subarvore desbalanceada seguida de uma rotacao
    esquerda no nodo raiz da subarv. desbalanceada, fica mais simples definir essa
    funcao utilizando as funcoes ja criadas:
    */
    d->dir = rotacao_direita(d->dir);
    return rotacao_esquerda(d);
}

/*
Para tornar a insercao mais rapida, utilizamos uma tecnica para evitar calculos em nodos
nao desbalanceados: visto que se ao inserirmos um nodo em uma arvore ela deixar de ser AVL,
necessitamos apenas de uma rotacao (simples ou dupla) para que ela se torne novamente AVL.
Para isso, precisamos de uma flag (no caso, a flag eh a var. balanceou). Como estamos
utilizando funcoes recursivas, precisaremos de uma funcao auxiliar para realizar esse metodo,
como segue:
*/
PtDic* insere_palavra_dic(PtDic *d, TipoPalavra p[MAX])
{
    int balanceou=0;
    return (insere_palavra_dic_r(d, p, &balanceou));
}

/*
Funcao auxiliar para inserir uma palavra na AVL
*/
PtDic* insere_palavra_dic_r(PtDic *d, TipoPalavra p[MAX],int *balanceou)
{
    int alt_esq, alt_dir, max;
    PtDic *tmp=NULL, *a=NULL, *b=NULL;

    //verifica se o nodo do dicionario esta vazio
    if (d == NULL)
    {
        //insere palavra no nodo atual
        d = (PtDic*) malloc(sizeof(PtDic));
        strcpy(d->palavra,p);
        d->esq = NULL;
        d->dir = NULL;
        d->altura = 0;
        return d;
    }
    //verifica se palavra a inserir eh MAIOR que a palavra do nodo atual
    else if (palavracmp(p,d->palavra) > 0)
    {
        d->dir = insere_palavra_dic_r(d->dir,p,balanceou);

        //verifica se alguma rotacao ja foi realizada
        if(!*balanceou)
        {
            alt_esq = get_altura(d->dir);
            alt_dir = get_altura(d->esq);

            //verifica se precisa balancear
            if(abs(alt_esq-alt_dir) >= 2)
            {
                tmp = d->dir;
                a = tmp->dir;
                b = tmp->esq;

                //Simples a esquerda
                if ( get_altura(a) >= get_altura(b) )
                {
                    d=rotacao_esquerda(d);
                }
                //Dupla a esquerda
                else
                {
                    d=rotacao_dupla_esquerda(d);
                }

                *balanceou = 1;
            }

            //atualiza a altura do nodo
            alt_esq = get_altura ( d->esq );
            alt_dir = get_altura ( d->dir );
            max = alt_esq > alt_dir ? alt_esq : alt_dir;

            //altura da raiz da subarvore eh a maior das alturas de suas subarv. + o proprio nodo
            d->altura = max + 1;
        }
    }
    //verifica se palavra a inserir eh MENOR que a palavra do nodo atual
    else if (palavracmp(p,d->palavra) < 0)
    {
        d->esq = insere_palavra_dic_r(d->esq,p,balanceou);

        //verifica se alguma rotacao ja foi realizada
        if(!*balanceou)
        {
            alt_esq = get_altura(d->dir);
            alt_dir = get_altura(d->esq);

            //verifica se precisa balancear
            if(abs(alt_esq-alt_dir) >= 2)
            {
                tmp = d->esq;
                a = tmp->dir;
                b = tmp->esq;

                //Simples a direita
                if ( get_altura(a) < get_altura(b) )
                {
                    d=rotacao_direita(d);
                }
                //Dupla a direita
                else
                {
                    d=rotacao_dupla_direita(d);
                }

                *balanceou = 1;
            }

            //atualiza a altura do nodo
            alt_esq = get_altura ( d->esq );
            alt_dir = get_altura ( d->dir );
            max = alt_esq > alt_dir ? alt_esq : alt_dir;

            //altura da raiz da subarvore eh a maior das alturas de suas subarv. + o proprio nodo
            d->altura = max + 1;
        }
    }

    return d;
}

/* retorna o parametro altura de um nodo. Retorna -1 se ele for nulo, 0 se for folha */
int get_altura(PtDic *d)
{
    return d==NULL ? -1 : d->altura;
}

/* Imprime as palavras como uma arvore na horizontal (melhor para arvores pequenas) */
void imprime_dicionario(PtDic *d, int altura)
{
    int i,nivel;

    if (d!= NULL)
    {
        imprime_dicionario(d->dir,altura);

        nivel = altura - calc_altura(d);
        for(i=0; i<nivel; i++)
            printf(" ");

        imprime_palavra(d->palavra);
        printf("\n");

        imprime_dicionario(d->esq,altura);
    }
}

/* destroi o dicionario liberando a memoria alocada para ele e suas palavras */
PtDic* destroi_dic(PtDic* d)
{
    //utiliza caminhamento pos-fixado a esq para liberar os nodos
    if (d != NULL)
    {
        destroi_dic(d->esq);
        destroi_dic(d->dir);
        //libera a memoria ocupada para o nodo
        free(d);
    }

    return NULL;
}
