#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tad_erros.h"
#include "tad_palavra.h"
#include "ansi_to_ascii.h"
#include "common.h"

PtErro* cria_lista(void)
{
    return NULL;
}

//Funo para incero
PtErro* insere_ord (PtErro *l, TipoPalavra p[])
{
    PtErro *novo; //novo elemento
    PtErro *ant = NULL; //ponteiro auxiliar para a posio anterior
    PtErro *ptaux = l; //ponteiro auxiliar para percorrer a lista
    int encontrou=0;

    /*aloca um novo nodo */
    novo = (PtErro*) malloc(sizeof(PtErro));

    /*insere a informao no novo nodo*/
    strcpy(novo->palavra,p);
    novo->freq=1;

    /* procura por palavra igual ja inserida */
    while(ptaux!=NULL && !encontrou)
    {
        //verifica se econtrou a palavra
        if(palavracmp(ptaux->palavra,p)==0)
        {
            //ptaux = palavra repetida
            encontrou=1;
        }
        else
        {
            ant=ptaux;
            ptaux=ptaux->prox;
        }
    }

    /* se ao encontrou repetida, insere em ordem de frequencia e depois alfabetica */
    if(!encontrou)
    {
        ptaux = l;

        /* procura posicao em que comecam os itens com frequencia = 1 */
        while(ptaux!=NULL && ptaux->freq>1)
        {
            ant = ptaux;
            ptaux = ptaux->prox;
        }

        /* procura a posicao respeitando a ordem alfabetica */
        while(ptaux!=NULL && palavracmp(ptaux->palavra,p)<0)
        {
            ant = ptaux;
            ptaux = ptaux->prox;
        }
    }

    //Palavra j existe na lista.
    if (encontrou==1)
    {
        //incrementa frequencia
        novo->freq=ptaux->freq+1;
        //remove o nodo ja existente
        l=remover(l,ptaux->palavra);
        //insere o nodo com a nova frequencia
        l=ord(l,novo);
    }
    /*o anterior no existe, logo o elemento ser inserido na primeira posio*/
    else if (ant == NULL)
    {
        novo->prox = l;
        l = novo;
    }
    /*elemento inserido no meio da lista*/
    else
    {
        novo->prox = ant->prox;
        ant->prox = novo;
    }

    return l;
}

//Funo para inserir nodos em ordem decrescente de frequencia
PtErro* ord(PtErro* l,PtErro* novo)
{
    PtErro* ptaux=l;
    PtErro* ant=NULL;

    while (ptaux!=NULL && (ptaux->freq>novo->freq))
    {
        ant = ptaux;
        ptaux = ptaux->prox;
    }

    /*o anterior no existe, logo o elemento ser inserido na primeira posio*/
    if (ant == NULL)
    {
        novo->prox = l;
        l = novo;
    }
    /*elemento inserido no meio da lista*/
    else
    {
        novo->prox = ant->prox;
        ant->prox = novo;
    }

return l;
}

PtErro* remover(PtErro* l, TipoPalavra pal[])
{
    PtErro *ant = NULL; //ponteiro auxiliar para a posio anterior
    PtErro *ptaux = l; //ponteiro auxiliar para percorrer a lista

    /*procura o elemento na lista*/
    while ((ptaux!=NULL) && (palavracmp(ptaux->palavra,pal)!=0))
    {
        ant = ptaux;
        ptaux = ptaux->prox;
    }

    /*verifica se achou*/
    if (ptaux == NULL)
        return l; /*retorna a lista original*/

    if (ant == NULL) /*vai remover o primeiro elemento*/
        l = ptaux->prox;
    else /*vai remover do meio ou do final*/
        ant->prox = ptaux->prox;

    free(ptaux); /*libera a memria alocada*/

    return l;
}

/*Imprime TODA a lista de palavras erradas*/
void imprime(PtErro* l)
{
    PtErro* ptaux;
    int i=1;

    if (l == NULL)
        printf("Nenhum erro encontrado!\n\n\n");
    else
    {
        printf("  K|                Erro| Frequencia\n");
        printf("------------------------------------\n");
        for (ptaux=l; ptaux!=NULL; ptaux=ptaux->prox)
        {
            printf("%3d|%20s|%d\n",i,ptaux->palavra,ptaux->freq);

            //pausa a tela quando for rolar
            if(i%22==0)
            {
                pause();
                clear_screen();
                printf("  K|                Erro| Frequencia\n");
                printf("------------------------------------\n");
            }

            i++;
        }
    }

}

/*Imprime TODA a lista de palavras erradas*/
void imprime_k(PtErro* l, int k)
{
    PtErro* ptaux;
    int i=1;

    ptaux=l;
    if (l == NULL)
        printf("Nenhum erro encontrado!\n");
    else
    {
        printf("  K|                Erro| Frequencia\n");
        printf("------------------------------------\n");

        while (i<=k && ptaux!=NULL)
        {
            printf("%3d|%20s|%d\n",i,ptaux->palavra,ptaux->freq);
            ptaux=ptaux->prox;

            //pausa a tela quando for rolar
            if(i%22==0)
            {
                pause();
                clear_screen();
                printf("  K|                Erro| Frequencia\n");
                printf("------------------------------------\n");
            }

            i++;
        }
    }
}

/*Destroi lista de palavras erradas*/
PtErro* destroi(PtErro* l)
{
    PtErro *ptaux; //ponteiro auxiliar para percorrer a lista
    while (l != NULL)
    {
        ptaux = l;
        l = l->prox;
        free(ptaux);
    }
    free(l);
    return NULL;
}
