#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "tad_palavra.h"
#include "ansi_to_ascii.h"
#include "extchar.h"

/* inicializa a palavra (lista de letras) */
void cria_palavra(TipoPalavra p[])
{
    p[0] = '\0';
}

/*
insere uma letra em uma palavra
 recebe l em ansi e insere em ascii
*/
int insere_letra(TipoPalavra p[], unsigned char l)
{
    int ultima_pos=0;

    //encontra a ultima posicao ocupada (somar 1 para contar o \0)
    ultima_pos = strlen(p);

    //checa se ha espaco para inserir a letra
    if(ultima_pos < MAX-1)
    {
        p[ultima_pos] = tolower(ansi_to_ascii(l));
        p[ultima_pos+1] = '\0';
    }
    else
    {
        //palavra ja esta cheia
        return 0;
    }

    return 1;
}

/* dado uma palavra com os caracteres */
void imprime_palavra(TipoPalavra p[])
{
    int i=0;

    while(p[i]!= '\0')
    {
        printf("%c",p[i]);
        i++;
    }
}

/* converte uma palavra de ansi (win-1252) para ascii (cp850) */
void converte_palavra_ascii(TipoPalavra p[])
{
    int i;

    for(i=0;i<strlen(p);i++)
        p[i] = ansi_to_ascii(p[i]);
}

/*
  compara duas palavras e possui o mesmo comportamento da strcmp, porem aceitando
  acentos (caracteres em CP-850)

  USO:
    Dadas duas palavras, retorna:
       >0: palavra 1 maior que palavra 2
       =0: palavras sao iguais
       <0: palavra 2 maior que palavra 1
*/
int palavracmp(TipoPalavra p1[], TipoPalavra p2[])
{
    int i=0;
    TipoPalavra p1_tmp, p2_tmp;
    TipoPalavra p1_sem_ac[MAX], p2_sem_ac[MAX];

    //encontra o caractere que difere
    while(p1[i]!='\0' && p2[i]!='\0' && tolowerextch(p1[i])==tolowerextch(p2[i]))
    {
        i++;
    }

    //passa os caracteres que diferem para minuscula
    p1_tmp = tolowerextch(p1[i]);
    p2_tmp = tolowerextch(p2[i]);

    //verifica se sao caracteres sem acento
    if(p1_tmp < 128 && p2_tmp < 128)
    {
        //retorna a diferenca entre as duas
        return p1_tmp - p2_tmp;
    }
    //uma delas ou as duas sao extendidas
    else
    {
        //remove os acentos das palavras para compara-las
        strcpy(p1_sem_ac,p1);
        strcpy(p2_sem_ac,p2);

        for(i=0;i<strlen(p1_sem_ac);i++)
            p1_sem_ac[i] = tolower(removeaccentsch(p1_sem_ac[i]));

        for(i=0;i<strlen(p2_sem_ac);i++)
            p2_sem_ac[i] = tolower(removeaccentsch(p2_sem_ac[i]));


        //verifica se sao palavras que diferem apenas por acentos (ex: esta e est)
        if(strcmp(p1_sem_ac,p2_sem_ac) == 0)
        {
            /*
            retorna a diferenca entre as duas letras:
            - primeira nao tem acento e segunda tem: retorna 1
            - primeira tem acento e segunda nao: retorna -1
            - se duas tiverem acento, retorna a diferenca deles conforme a ordem das
            letras na tabela (varia para cada vogal, mas mantem um padrao entre cada letra)
            */
            return p2_tmp - p1_tmp;

        }
        //sao palavras com letras diferentes, ignorar os acentos
        else
        {
            //retorna a diferenca entre as duas, descartando os acentos
            return strcmp(p1_sem_ac,p2_sem_ac);
        }
    }
}

/*
verifica se um dado caractere eh uma letra valida
baseado na tabela da codificacao windows-1252 (ANSI)
Tabela de caracteres:
http://en.wikipedia.org/wiki/Windows-1252
*/
int verifica_letra(unsigned char letra)
{
    // verifica pelo codigo da letra
    if(letra < 65)
        return 0;
    else if (
        (letra>=97 && letra<=122) ||
        (letra>=65 && letra<=90) ||
        (letra>=224 && letra<=228) ||
        (letra>=231 && letra<=239) ||
        (letra>=241 && letra<=246) ||
        (letra>=249 && letra<=253) ||
        (letra>=192 && letra<=196) ||
        (letra>=199 && letra<=207) ||
        (letra>=209 && letra<=214) ||
        (letra>=217 && letra<=221) ||
        (letra==159 || letra==255))
        return 1;

    return 0;
}
